@@ -5,19 +5,26 @@
 
 from abc import ABC, abstractmethod
 from typing import Iterator, Union
-import fcntl
 import json
 import logging
 import socket
 import ssl
-import struct
-import termios
 import time
 
+
+# Use fcntl to control socket buffering if available
+try:
+    import fcntl
+    import struct
+    import termios
+except ImportError:
+    fcntl = None
+
 from requests.auth import HTTPBasicAuth
 import pylogbeat
 import requests
 
+from logstash_async.constants import constants
 from logstash_async.utils import ichunked
 
 
@@ -126,14 +133,27 @@ class UdpTransport:
     def _close(self, force=False):
         if not self._keep_connection or force:
             if self._sock:
-                while not self._is_sock_write_buff_empty():
-                    time.sleep(0.05)
+                self._wait_for_socket_buffer_empty()
                 self._sock.shutdown(socket.SHUT_WR)
                 self._sock.close()
                 self._sock = None
 
+    # ----------------------------------------------------------------------
+    def _wait_for_socket_buffer_empty(self):
+        wait_timeout = constants.SOCKET_CLOSE_WAIT_TIMEOUT
+        interval = 0.05
+        time_waited = 0
+        # wait until the socket's write buffer is empty
+        # but do not wait longer than SOCKET_CLOSE_WAIT_TIMEOUT
+        while not self._is_sock_write_buff_empty() and time_waited < wait_timeout:
+            time_waited += interval
+            time.sleep(interval)
+
     # ----------------------------------------------------------------------
     def _is_sock_write_buff_empty(self):
+        if fcntl is None:
+            return True
+
         socket_fd = self._sock.fileno()
         buffer_size = struct.pack('I', 0)
         ioctl_result = fcntl.ioctl(socket_fd, termios.TIOCOUTQ, buffer_size)
